# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Jens Wilberg
#
# This file is part of wordrating.
#
# wordrating is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wordrating is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wordrating.  If not, see <http://www.gnu.org/licenses/>.
"""Module for data storage."""
import os
import sys
import csv
import codecs
import shutil
import json


class Database():
    """Class for data and file-operations."""

    def __init__(self):
        """Class for data storage."""
        tmp = os.path.dirname(os.path.realpath(__file__))
        self._path = os.path.join(tmp, "data")
        self._scores = []  # list with words sorted by scores
        self._new_words = []
        self._words = []
        self._t = {}
        self._index = -1
        self._in_scores = []
        self._read_scores()
        self._read_words()
        self._load_progress()

    def _read_scores(self):
        """Read all scores."""
        scores = []
        if sys.version_info >= (3, 0):
            with codecs.open(os.path.join(self._path, "lexicon-scores.csv"),
                             "r", "utf-8") as csvfile:
                reader = csv.DictReader(csvfile)
                for row in reader:
                    if row[''] == "t":
                        self._t = {
                            "norm": float(row["norm"])
                        }
                        continue
                    tmp = {
                        "word": row[''],
                        "norm": float(row["norm"])
                    }
                    scores.append(tmp)
        else:
            # Python2 compatibility
            scores = []
            with open(os.path.join(self._path, "lexicon-scores.csv"),
                      "r") as csvfile:
                reader = csv.DictReader(csvfile)
                for row in reader:
                    if row[''] == "t":
                        self._t = {
                            "norm": float(row["norm"])
                        }
                        continue
                    tmp = {
                        "word": unicode(row[''], "utf-8"),
                        "norm": float(row["norm"])
                    }
                    scores.append(tmp)
        self._scores = sorted(scores, key=lambda k: k['norm'])

    def _read_words(self):
        """Reads the new words."""
        try:
            with codecs.open(os.path.join(self._path, "word.list"), "r",
                             "utf-8") as f:
                for line in f:
                    word = line.replace('\n', '')
                    if word == "":
                        continue
                    self._new_words.append(word)
                    if any(d["word"] == word for d in self._scores):
                        self._in_scores.append(word)
        except:
            pass

    def check_words(self):
        """Checks if there are words to rate.

        Returns
        -------
        bool
            True if there are words
        """
        return True if len(self._new_words) else False

    def read_new_words(self, path):
        """Reads the list of the new words to rate.

        Parameters
        ----------
        path : str
            The path to the word list.
        """
        self._remove_old()
        shutil.copyfile(path,
                        os.path.join(self._path, "word.list"))
        self._read_words()

    def get_score(self, index):
        """Returns the Scoreobject at index.

        Parameters
        ----------
        index : int
            Index of the Scoreoject.

        Returns
        -------
        Scoreobject
            {'word': 'word', 'norm': float}
        """
        return self._scores[index]

    def get_score_list(self):
        """Returns sentiment lexicon.

        Returns
        -------
        list
            sentiment lexicon
        """
        return self._scores

    def get_score_length(self):
        """Returns length of sentiment lexicon.

        Returns
        -------
        int
            length of the lexicon
        """
        return len(self._scores)

    def next(self):
        """Returns the next word to rate.

        Returns
        -------
        str
            next word to rate
        """
        self._index += 1
        new_word = self._new_words[self._index]
        return new_word

    def save_CSV(self, objList, fieldnames, filename="result.csv"):
        """Saves the given object-list as csv.

        Parameters
        ----------
        objList : list
            list with objects.
        fieldnames : list
            List with fieldnames.
        """
        new_file = False
        if not os.path.exists(os.path.join(self._path, filename)):
            new_file = True
        if sys.version_info >= (3, 0):
            with codecs.open(os.path.join(self._path, filename), "a",
                             "utf-8") as csvfile:
                writer = csv.DictWriter(
                    csvfile, fieldnames=fieldnames, delimiter=';')
                if new_file:
                    writer.writeheader()
                    new_file = False
                writer.writerows(objList)
        else:
            # Python2 compatibility
            with open(os.path.join(self._path, filename), "a") as csvfile:
                writer = csv.DictWriter(
                    csvfile, fieldnames=fieldnames, delimiter=';')
                if new_file:
                    writer.writeheader()
                    new_file = False
                for row in objList:
                    try:
                        row["Wort1"] = row["Wort1"].encode('utf-8')
                        row["Wort2"] = row["Wort2"].encode('utf-8')
                    except TypeError:
                        pass
                    writer.writerow(row)

    def save_progress(self):
        """Saves progress.

        Parameters
        ----------
        rand_width : int
            Random width
        """
        tmp = {"i": self._index}
        try:
            with codecs.open(os.path.join(self._path, "config"), "r",
                             "utf-8") as f:
                tmp = json.load(f)
                tmp["i"] = self._index
        except IOError:
            # If no progress is present
            pass
        with codecs.open(os.path.join(self._path, "config"), "w+",
                         "utf-8") as of:
            json.dump(tmp, of)

    def save_random_width(self, random_width):
        """Saves progress.

        Parameters
        ----------
        rand_width : int
            Random width
        """
        tmp = {"rw": random_width}
        try:
            with codecs.open(os.path.join(self._path, "config"), "r",
                             "utf-8") as f:
                tmp = json.load(f)
                tmp["rw"] = random_width
        except IOError:
            # If no progress is present
            pass
        with codecs.open(os.path.join(self._path, "config"), "w+",
                         "utf-8") as of:
            json.dump(tmp, of)

    def get_random_width(self):
        """Returns the random width.

        Returns
        -------
        int
            Random width
        """
        try:
            with codecs.open(os.path.join(self._path, "config"), "r",
                             "utf-8") as f:
                tmp = json.load(f)
                return tmp["rw"]
        except IOError:
            return 0

    def _load_progress(self):
        """Load progress."""
        try:
            with codecs.open(os.path.join(self._path, "config"), "r",
                             "utf-8") as f:
                tmp = json.load(f)
                try:
                    self._index = tmp["i"]
                except KeyError:
                    pass
        except IOError:
            # If no progress is present
            pass

    def _remove_old(self):
        """Removes the old Progress."""
        self._index = -1
        self._new_words = []
        self._in_scores = []
        try:
            os.remove(os.path.join(self._path, "word.list"))
        except OSError:
            pass
        try:
            os.remove(os.path.join(self._path, "result.csv"))
        except OSError:
            pass
        try:
            os.remove(os.path.join(self._path, "config"))
        except OSError:
            pass

    def reusult(self, path):
        """Moves the reult to the workingdirectory."""
        shutil.copyfile(os.path.join(self._path, "result.csv"), path)
        self._remove_old()

    def check_in_scores(self):
        """Checks if some of the new words are in the sentiment lexicon.

        Returns
        -------
        bool
            True if some of the new words in the lexicon
        """
        return bool(len(self._in_scores))

    def remove_in_scores(self):
        """Removes the new words which are in the score-list."""
        self._new_words = [
            item for item in self._new_words if item not in self._in_scores]
        self._in_scores = []
        self._index = -1
        with codecs.open(os.path.join(self._path, "word.list"), "w+",
                         "utf-8") as of:
            of.write('\n'.join(self._new_words) + '\n')
