# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Jens Wilberg
#
# This file is part of wordrating.
#
# wordrating is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wordrating is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wordrating.  If not, see <http://www.gnu.org/licenses/>.
"""GUI to rate new words."""
import wx
import os
from wordrating import silverstein, __version__
# If wxPython Phoenix is the current wxPython version
if "phoenix" in wx.version():
    import wx.adv

_ = wx.GetTranslation


class RandWidthDialog(wx.Dialog):
    """Class of Randomwidth Dialog.

    Parameters
    ----------
    parent : wx.Window
        Can be None, a frame or another dialog box
    id : wx.WindowID
        An identifier for the dialog. A value of -1 is taken to mean a default
    initial : int
        Defaultvalue for the random width
    """

    def __init__(self, parent=None, id=wx.ID_ANY, initial=0):
        """Constructor."""
        style = wx.DEFAULT_DIALOG_STYLE
        super(RandWidthDialog, self).__init__(parent, id, _("Random Width"),
                                              style=style)
        self._input = wx.SpinCtrl(self, wx.ID_ANY, min=0, initial=initial)
        message = _("Please choose your appropriated random width")

        # Layout
        topsizer = wx.BoxSizer(wx.VERTICAL)
        topsizer.Add(self.CreateTextSizer(message), 1, wx.ALL, 10)
        randsizer = wx.BoxSizer(wx.HORIZONTAL)
        randsizer.AddSpacer(10)
        randsizer.Add(self.CreateTextSizer(
            _("Random width: ")), 1, wx.CENTER, 5)
        randsizer.Add(self._input, 0, wx.ALL, 5)
        topsizer.Add(randsizer)
        buttonSizer = self.CreateButtonSizer(wx.OK)
        topsizer.Add(buttonSizer, 1, wx.ALL | wx.EXPAND, 10)
        self.SetAutoLayout(True)
        self.SetSizer(topsizer)
        topsizer.SetSizeHints(self)
        topsizer.Fit(self)
        self.Center(wx.BOTH)
        self._input.SetFocus()

    def OnClose(self, e):
        """Close Dialog."""
        self.Destroy()

    def GetValue(self):
        """Returns the Value of the input-field.

        Returns
        -------
        int
            value of input
        """
        return self._input.GetValue()


class MainPanel(wx.Panel):
    """Class of the main Panel.

    The MainPanel takes the same arguments as the wx.Panel class.
    """

    def __init__(self, ss, *args, **kwargs):
        """Constructor."""
        wx.Panel.__init__(self, *args, **kwargs)
        self.ss = ss

        self.fin = False
        pair = self.ss.get_word_pair()

        # create widgets
        quest = wx.StaticText(
            self, wx.ID_ANY,
            _("Which word is more positive (less negative)?"),
            style=wx.ALIGN_CENTRE)
        tip = wx.StaticText(
            self, wx.ID_ANY,
            _("You can also use keys: left word=1; right word=2; draw=0 "
              "(use Enter to accept)"),
            style=wx.ALIGN_CENTRE)

        self.word1 = wx.Button(self, wx.ID_ANY, pair[0])
        self.word2 = wx.Button(self, wx.ID_ANY, pair[1])
        self.eq = wx.Button(self, wx.ID_ANY, _("draw"))
        self.back = wx.Button(self, wx.ID_ANY, _("&back"))

        # change Font
        font = wx.Font(22, wx.MODERN, wx.NORMAL, wx.NORMAL)
        quest.SetFont(font)
        font = wx.Font(16, wx.MODERN, wx.NORMAL, wx.NORMAL)
        self.word1.SetFont(font)
        self.word2.SetFont(font)
        font = wx.Font(14, wx.MODERN, wx.ITALIC, wx.NORMAL)
        self.eq.SetFont(font)

        # eventhandler
        table = []
        i = 0
        for func in [self.selectD, self.selectW1, self.selectW2]:
            eventId = wx.NewId()
            self.Bind(wx.EVT_MENU, func, id=eventId)
            table.append((wx.ACCEL_NORMAL, 48 + i, eventId))
            table.append((wx.ACCEL_NORMAL, 324 + i, eventId))
            i += 1
        i = 0
        funcs = [self.onWin, self.onLose, self.onDraw]
        for button in [self.word1, self.word2, self.eq]:
            button.Bind(wx.EVT_BUTTON, funcs[i])
            button.Bind(wx.EVT_KILL_FOCUS, self.loseFocus)
            button.Bind(wx.EVT_SET_FOCUS, self.getFocus)
            i += 1
        eventId = wx.NewId()
        table.append((wx.ACCEL_CTRL, ord('Z'), eventId))
        self.Bind(wx.EVT_MENU, self.onBack, id=eventId)
        self.back.Bind(wx.EVT_BUTTON, self.onBack)
        self.back.Disable()
        accTable = wx.AcceleratorTable(table)
        self.SetAcceleratorTable(accTable)

        # layout
        topSizer = wx.BoxSizer(wx.VERTICAL)
        titleSizer = wx.BoxSizer(wx.HORIZONTAL)
        wordSizer = wx.BoxSizer(wx.HORIZONTAL)

        titleSizer.Add(quest, 0, wx.ALL, 20)

        wordSizer.Add(self.word1, 1, wx.LEFT | wx.ALL | wx.EXPAND, 20)
        wordSizer.AddSpacer(20)
        wordSizer.Add(self.word2, 1, wx.RIGHT | wx.ALL | wx.EXPAND, 20)

        topSizer.Add(wx.StaticLine(self), 0, wx.ALL | wx.EXPAND, 5)
        topSizer.Add(titleSizer, 0, wx.CENTER)
        topSizer.Add(wordSizer, 0, wx.CENTER, 5)
        topSizer.Add(self.eq, 0, wx.CENTER | wx.ALL, 5)
        topSizer.Add(tip, 0, wx.CENTER | wx.BOTTOM)
        topSizer.Add(self.back, 0, wx.EXPAND | wx.ALL)

        self.SetSizerAndFit(topSizer)
        self.Parent.Layout()

        self.SetFocus()

    def selectW1(self, event):
        """Selects the first word."""
        self.word1.SetFocus()

    def selectW2(self, event):
        """Selects the second word."""
        self.word2.SetFocus()

    def selectD(self, event):
        """Selects draw."""
        self.eq.SetFocus()

    def getFocus(self, event):
        """If button gets focus."""
        if not os.name == "nt":
            event.GetEventObject().SetBackgroundColour('#00a8e6')

    def loseFocus(self, event):
        """If button loses focus."""
        btn = event.GetEventObject()
        if not os.name == "nt":
            btn.SetBackgroundColour(
                wx.SystemSettings.GetColour(wx.SYS_COLOUR_BACKGROUND))

    def onWin(self, event):
        """New word wins."""
        self.compare(1.0)

    def onLose(self, event):
        """New word loses."""
        self.compare(0.0)

    def onDraw(self, event):
        """Draw."""
        self.compare(0.5)

    def compare(self, win):
        """Saves the result of the comprehension.

        Parameters
        ----------
        win : float
            value of win (win = 1.0, draw = 0.5, lose = 0.0)
        """
        try:
            self.ss.compare_words(win)
            self.changeWords()
        except IndexError:
            self.completed()

    def onBack(self, event):
        """Undos the rating for the current word."""
        if self.back.IsEnabled():
            self.ss.undo()
            self.changeWords()

    def changeWords(self):
        """Changes the displayed words."""
        if self.ss.check_last():
            self.back.Enable()
        else:
            self.back.Disable()
            self.word1.SetFocus()
        pair = self.ss.get_word_pair()
        self.word1.SetLabel(pair[0])
        self.word2.SetLabel(pair[1])
        self.Parent.Layout()

    def completed(self):
        """Complete the rating."""
        self.fin = True
        parent = self.GetParent()
        parent.showDlg(self, _("Rating complete."), _("Complete"), wx.OK)
        saveDlg = wx.FileDialog(self, _("Save Results"), "results.csv", "",
                                "All (*)|*",
                                wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if saveDlg.ShowModal() == wx.ID_OK:
            path = saveDlg.GetPath()
            self.ss.result(path)
        else:
            parent.showDlg(self, _(
                "You can save your result next time when you continue."),
                _("Attention"), wx.OK)
        parent.Close(True)


class MainFrame(wx.Frame):
    """Class for the MainFrame.

    The MainFrame takes the same arguments as the wx.Frame class.
    """

    def __init__(self, arg=None, randWidth=0, *args, **kwargs):
        """Constructor."""
        wx.Frame.__init__(self, *args, **kwargs)
        self.CreateStatusBar()
        self.panel = None
        self.startPanel = wx.Panel(self, wx.ID_ANY)
        self.ss = silverstein.Silverstein()
        self._randomWidth = randWidth

        # check for command line arguments
        if arg:
            self.ss.read_new_words(arg)
            if self.ss.check_in_scores():
                msg = _(
                    "There are words in your word-list, which are in the "
                    "score-list.\nDo you want to delete them?")
                retval = self.showDlg(self, msg, _(
                    "Attention"), wx.YES_NO | wx.YES_DEFAULT |
                    wx.ICON_EXCLAMATION)
                if retval == wx.ID_YES:
                    try:
                        self.ss.remove_in_scores()
                    except IndexError:
                        self.showDlg(self, _("File is empty."), _(
                            "Attention"), wx.OK | wx.ICON_EXCLAMATION)
                        self.Destroy()
                        return
            self.ss.set_random_width(self._randomWidth)
            self.panel = MainPanel(self.ss, self, wx.ID_ANY)

        # create menu
        filemenu = wx.Menu()
        helpmenu = wx.Menu()
        menuExit = filemenu.Append(wx.ID_EXIT, "Exit", _(" Exit Program"))
        menuAbout = helpmenu.Append(wx.ID_ABOUT,
                                    _("About"), _(" Infos about this program"))

        # create menubar
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu, _("&File"))
        menuBar.Append(helpmenu, _("&Help"))
        self.SetMenuBar(menuBar)

        # Eventhandler
        self.Bind(wx.EVT_MENU, self.onExit, menuExit)
        self.Bind(wx.EVT_MENU, self.onAbout, menuAbout)
        self.Bind(wx.EVT_CLOSE, self.onExit)

        sizer = wx.BoxSizer(wx.HORIZONTAL)

        # when there is no command line argument or it is wrong, the startpage
        # will be created. Otherweise the rating starts.
        if self.panel is None:
            start = wx.Button(self.startPanel, wx.ID_ANY,
                              _("&New"), name="start")
            con = wx.Button(self.startPanel, wx.ID_ANY,
                            _("&Continue"), name="con")
            con.SetFocus()
            start.Bind(wx.EVT_BUTTON, self.onButton)
            con.Bind(wx.EVT_BUTTON, self.onButton)
            if not self.ss.check_words():
                # disable continue button
                con.Disable()
                start.SetFocus()
            buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
            buttonSizer.Add(con, 1, wx.EXPAND)
            buttonSizer.AddSpacer(20)
            buttonSizer.Add(start, 1, wx.EXPAND)
            self.startPanel.SetSizerAndFit(buttonSizer)
            sizer.Add(self.startPanel, 1, wx.EXPAND)
        else:
            self.startPanel.Hide()
            sizer.Add(self.panel, 1, wx.EXPAND)

        self.SetSizerAndFit(sizer)

    def onExit(self, event):
        """Handle closeevent."""
        if self.panel:
            if self.panel.fin:
                # If the rating is completed, the program can be closed
                self.Destroy()
                return
            ret = self.showDlg(self,
                               _("If you close the program only the last word "
                                 "you completed will be saved!"),
                               _("Attention"), wx.OK | wx.CANCEL |
                               wx.ICON_EXCLAMATION)
            if ret == wx.ID_OK:
                self.Destroy()
        else:
            # if there is no panel, the program can be closed without warning
            self.Destroy()

    def onAbout(self, event):
        """Shows informations about this program."""
        description = _("""wordrating is a GUI-Tool to rate words.
The new words are compared to words where the scores are already known.
The Comparison is saved in a csv file, so you can compute the scores.""")

        licence = """wordrating is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

wordrating is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see http://www.gnu.org/licenses/."""

        info = None
        if "phoenix" in wx.version():
            info = wx.adv.AboutDialogInfo()  # wxPython Phoenix
        else:
            info = wx.AboutDialogInfo()  # wxPython classic
        info.SetName('wordrating')
        info.SetVersion(__version__.VERSION_STRING)
        info.SetDescription(description)
        info.SetCopyright('Copyright (C) 2016 Jens Wilberg')
        info.SetWebSite(
            'http://informatik.hsnr.de/~dalitz/data/sentimentlexicon/')
        info.SetLicence(licence)
        info.AddDeveloper("Jens Wilberg")
        info.AddDocWriter("Jens Wilberg")
        info.AddTranslator("Jens Wilberg")
        if "phoenix" in wx.version():
            wx.adv.AboutBox(info)  # wxPython Phoenix
        else:
            wx.AboutBox(info)  # wxPython classic

    def onButton(self, event):
        """Handels click on startbtn."""
        name = event.GetEventObject().GetName()

        if name == "start":
            if self.ss.check_words():
                ret = self.showDlg(self, _("Your old progress will be "
                                           "overwritten.\nContinue?"),
                                   _("Attention"), wx.OK | wx.CANCEL |
                                   wx.ICON_EXCLAMATION)
                if ret == wx.ID_CANCEL:
                    return
            openFileDialog = wx.FileDialog(self, _("Choose Wordlist"), "", "",
                                           "Text (*.txt)|*.txt",
                                           wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

            if openFileDialog.ShowModal() == wx.ID_OK:
                path = openFileDialog.GetPath()
                self.ss.read_new_words(path)
                if self.ss.check_in_scores():
                    msg = _(
                        "There are words in your word-list, which are in the "
                        "score-list.\nDo you want to delete them?")
                    retval = self.showDlg(self, msg, _(
                        "Attention"), wx.YES_NO | wx.YES_DEFAULT)
                    if retval == wx.ID_YES:
                        try:
                            self.ss.remove_in_scores()
                        except IndexError:
                            self.showDlg(self, _("File is empty."), _(
                                "Attention"), wx.OK | wx.ICON_EXCLAMATION)
                            self.Destroy()
                            return
                openFileDialog.Destroy()
                randDlg = RandWidthDialog(self, wx.ID_ANY, self._randomWidth)
                randDlg.ShowModal()
                randWidth = randDlg.GetValue()
                randDlg.Destroy()
                self.startPanel.Hide()
                self.ss.set_random_width(randWidth)
                self.panel = MainPanel(self.ss, self, wx.ID_ANY)
                sizer = wx.BoxSizer(wx.VERTICAL)
                sizer.Add(self.panel, 1, wx.EXPAND)
                self.SetSizerAndFit(sizer)
            else:
                openFileDialog.Destroy()

        elif name == "con":
            self.startPanel.Hide()
            try:
                self.ss.new_word()
                self.panel = MainPanel(self.ss, self, wx.ID_ANY)
                sizer = wx.BoxSizer(wx.VERTICAL)
                sizer.Add(self.panel, 1, wx.EXPAND)
                self.SetSizerAndFit(sizer)
            except IndexError:
                self.showDlg(self, _("Rating already completed."),
                             _("Complete"), wx.OK)
                saveDlg = wx.FileDialog(self, _("Save Results"), "results.csv",
                                        "", "All (*)|*", wx.FD_SAVE |
                                        wx.FD_OVERWRITE_PROMPT)
                if saveDlg.ShowModal() == wx.ID_OK:
                    path = saveDlg.GetPath()
                    self.ss.result(path)
                    self.Destroy()
                    return
                self.startPanel.Show()

        self.Center()

    def showDlg(self, parent, message, title, style):
        """Displays a dialog.

        Arguments:
            parent:     The parent window of the dialoge
            message:    (str) The message that is shown
            title:      (str) The Title of the dialoge
            style:      (int) The style of the dialoge
        """
        dlg = wx.MessageDialog(parent=parent, message=message,
                               caption=title, style=style)
        retval = dlg.ShowModal()
        dlg.Destroy()
        return retval


def run(arg=None, random_width=0):
    """Run the Application.

    Arguments
    ---------
    arg : str
        The commandlineargumend which detremines the path to the wordlist.
    """
    app = wx.App()
    # locale
    appdir = os.path.dirname(os.path.abspath(__file__))
    localedir = os.path.join(appdir, "locale")
    localedomain = "wordrating"
    mylocale = wx.Locale(wx.LANGUAGE_DEFAULT)
    mylocale.AddCatalogLookupPathPrefix(localedir)
    mylocale.AddCatalog(localedomain)
    frame = MainFrame(parent=None, title="wordrating",
                      arg=arg, randWidth=random_width)
    app.SetTopWindow(frame)
    frame.Show()
    app.MainLoop()


if __name__ == "__main__":
    run()
