# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Jens Wilberg
#
# This file is part of wordrating.
#
# wordrating is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wordrating is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wordrating.  If not, see <http://www.gnu.org/licenses/>.
"""Implementation of the silverstein-algorithm."""
import math
from random import randint
from wordrating import database


class Silverstein():
    """Class which implements the silverstein-algorithm from Dalitz."""

    def __init__(self):
        """Init Silverstein."""
        self._db = database.Database()
        self._sdnorm = 1 / math.sqrt(3)
        # Choose a random word in +/- width from the found index
        self._random_width = self._db.get_random_width()
        # self.new_word()

    def _compare_words(self, win):
        """Calculates the index for the next word.

        This function calculates the index of the next Word.
        This word will be compared with the new word. The index is
        retrieved by the sum of all wins and draws of the comparison
        from the preceded words with the new word. The wins, draws and
        loses have the following values:
        win => 1.0
        draw => 0.5
        loses => 0.0

        Parameters
        ----------
        win : float
            Value for win, draw or lose.
        """
        self._save_last()
        self._level += 1
        # Words that are already compared
        self._words_done.append(self._word2["word"])
        self._q.append(self._word2["norm"])
        self._S += win
        self._m += 1
        self._lastresult = win

        if (self._index == 0) or \
                (self._index == self._db.get_score_length() - 1):
            # If the index reached the upper or lower bound, the seach
            # for a new index will be aborted and the first guess
            # will be calculated.
            self._compute_first_guess()

            # The neighbour with the nearest Score to the first guess will be
            # the next word for the comparison with the new word.
            self._get_next_neighbour()

            # the index must be invalid, so this function will not be executed
            # anymore
            self._index = -1
            return
        if self._lastresult <= 0.5:
            self._right = self._index
        else:
            self._left = self._index

        self._index = int((float(self._left) + float(self._right)) /
                          float(2)) + randint(-self._random_width,
                                              self._random_width)
        if not self._check_index():
            # If the index is invalid, the seach for a new index
            # will be aborted and the first guess will be calculated.
            self._compute_first_guess()

            # The neighbour with the nearest Score to the first guess will be
            # the next word for the comparison with the new word.
            self._get_next_neighbour()
        else:
            # The next word for the comparison is the word at index.
            self._word2 = self._db.get_score(self._index)

    def _check_index(self):
        """Check if the index is valid.

        index is valid when it is between the left bound and right bound,
        or the index is 0 or the length of the scorelist.

        Returns
        -------
        bool
            True if index is valid
        """
        if ((self._index > self._left) and (self._index < self._right)):
            return True
        else:
            return False

    def _compute_first_guess(self):
        """Calculate the first guess for the new word."""
        # First Guess (r0)
        r0 = (self._db.get_score(self._left)['norm'] +
              self._db.get_score(self._right)['norm']) / 2
        # Create a dict wihch is sorted by the differences betwen the scores
        # from the scorelist and the first guess.
        scores = self._db.get_score_list()
        diff = []
        for i in range(0, len(scores)):
            item = scores[i]
            # here the difference betwen the score at i and the first guess is
            # calculated
            tpl = ((item['norm'] - float(r0))**2, i)
            diff.append(tpl)
        self._diff = sorted(diff, key=lambda k: k[0])  # sort the diff
        self._ni = 0

    def _get_next_neighbour(self):
        """Chooses the nearest neighbour."""
        self._j += 1
        i = self._diff[self._j][1]
        self._word2 = self._db.get_score(i)
        while self._word2['word'] in self._words_done:
            # If the word was alread compared, the next word will be choosen
            self._j += 1
            i = self._diff[self._j][1]
            self._word2 = self._db.get_score(i)
        self._words_done.append(self._word2["word"])

    def _compare_neighbour(self, win):
        """Compares the word with its neighbour.

        win => 1.0
        draw => 0.5
        loses => 0.0

        Parameters
        ----------
        win : float
            if the word won
        """
        self._save_last()
        self._level += 1
        self._words_done.append(self._word2["word"])
        self._q.append(self._word2["norm"])
        self._S += win
        self._m += 1
        # The number of compared neighbours is counted. When 7 neighbour are
        # compared,the result will be saved and a new word without score will
        # be choosen.
        self._ni += 1
        if self._ni >= 7:
            fieldnames = ["Wort1", "Wort2", "Sieg1", "Unentschieden", "Sieg2"]
            self._db.save_progress()
            self._db.save_CSV(self._result, fieldnames)
            self.new_word()
            return
        self._get_next_neighbour()

    def _save_last(self):
        """Saves the last progress, so you can undo it."""
        tmp = {
            "left": self._left,
            "right": self._right,
            "index": self._index,
            "level": self._level,
            "m": self._m,
            "S": self._S,
            "lastresult": self._lastresult,
            "ni": self._ni,
            "j": self._j
        }
        self._last.append(tmp)

    def set_random_width(self, random_width):
        """Sets the random width.

        Parameters
        ----------
        random_width : int
            random width
        """
        self._random_width = random_width
        self._db.save_random_width(random_width)

    def new_word(self):
        """Reset all attributes."""
        self._last = []
        self._word = self._db.next()  # the new word without score
        self._left = 0
        self._right = self._db.get_score_length() - 1
        self._index = int((float(self._left) + float(self._right)) /
                          float(2)) + randint(-self._random_width,
                                              self._random_width)
        self._word2 = self._db.get_score(self._index)
        self._words_done = []
        self._level = 0
        self._m = 0
        self._S = 0.0
        self._q = []
        self._lastresult = 0.5
        self._diff = []
        self._ni = -1  # neighbour counter
        self._j = -1
        self._result = []

    def compare_words(self, win):
        """Compares two words.

        This function checks the index and chooses the appropriated
        compare function. The wins, draws and loses have the following
        values:
            win => 1.0
            draw => 0.5
            loses => 0.0
        Also the comprehension is saved.

        Parameters
        ----------
        win : float
            Value for win, draw or lose.
        """
        if self._check_index():
            self._compare_words(win)
        else:
            self._compare_neighbour(win)
        tmp = [0, 0, 0]
        if win == 1.0:
            tmp[0] = 1
        elif win == 0.5:
            tmp[1] = 1
        else:
            tmp[2] = 1
        obj = {
            "Wort1": self._word,
            "Wort2": self._word2["word"],
            "Sieg1": tmp[0],
            "Unentschieden": tmp[1],
            "Sieg2": tmp[2]
        }
        self._result.append(obj)

    def get_word_pair(self):
        """Returns the Pair of words, that are compared."""
        return (self._word, self._word2["word"])

    def result(self, path):
        """Copies the result in the workingdirectory."""
        self._db.reusult(path)

    def check_words(self):
        """Checks if there are words to rate.

        Returns
        -------
        bool
            True if there are more words to rate
        """
        return True if self._db.check_words() else False

    def read_new_words(self, path):
        """Reads the list of the new words to rate.

        Parameters
        ----------
        path : str
            The path to the word list.
        """
        self._db.read_new_words(path)
        self.new_word()

    def check_last(self):
        """Checks if something in last.

        Returns
        -------
        bool
            True if ther is something in last
        """
        return True if len(self._last) > 0 else False

    def undo(self):
        """Undos your last rating."""
        last = self._last.pop()
        self._words_done.pop()
        try:
            self._q.pop()
        except IndexError:
            pass
        try:
            self._result.pop()
        except IndexError:
            pass
        self._left = last["left"]
        self._right = last["right"]
        self._index = last["index"]
        self._level = last["level"]
        self._m = last["m"]
        self._S = last["S"]
        self._lastresult = last["lastresult"]
        self._ni = last["ni"]
        self._j = last["j"]
        if self._ni == -1:
            self._word2 = self._db.get_score(self._index)
        else:
            i = self._diff[self._j][1]
            self._word2 = self._db.get_score(i)

    def check_in_scores(self):
        """Checks if some of the new words are in the sentiment lexicon.

        Returns
        -------
        bool
            True if the new words are in the lexicon
        """
        return self._db.check_in_scores()

    def remove_in_scores(self):
        """Removes the new words which are in the score-list."""
        self._db.remove_in_scores()
        self.new_word()
